@@ -31,6 +31,7 @@
 #include "notification.h"
 #include "pmc_common.h"
 #include "print.h"
+#include "sad.h"
 #include "tlv.h"
 #include "uds.h"
 #include "util.h"
@@ -169,6 +170,7 @@ static void pmc_show(struct ptp_message *msg, FILE *fp)
 	struct subscribe_events_np *sen;
 	struct port_properties_np *ppn;
 	struct port_hwclock_np *phn;
+	struct cmlds_info_np *cmlds;
 	struct timePropertiesDS *tp;
 	struct management_tlv *mgt;
 	struct time_status_np *tsn;
@@ -198,7 +200,15 @@ static void pmc_show(struct ptp_message *msg, FILE *fp)
 	fprintf(fp, "\t%s seq %hu %s ",
 		pid2str(&msg->header.sourcePortIdentity),
 		msg->header.sequenceId, pmc_action_string(action));
-	if (msg_tlv_count(msg) != 1) {
+	switch (msg_tlv_count(msg)) {
+	case 1:
+		break;
+	case 2:
+		extra = TAILQ_LAST(&msg->tlv_list, tlv_list);
+		if (extra->tlv->type == TLV_AUTHENTICATION) {
+			break;
+		}
+	default:
 		goto out;
 	}
 	extra = TAILQ_FIRST(&msg->tlv_list);
@@ -449,12 +459,16 @@ static void pmc_show(struct ptp_message *msg, FILE *fp)
 	case MID_SUBSCRIBE_EVENTS_NP:
 		sen = (struct subscribe_events_np *) mgt->data;
 		fprintf(fp, "SUBSCRIBE_EVENTS_NP "
-			IFMT "duration          %hu"
-			IFMT "NOTIFY_PORT_STATE %s"
-			IFMT "NOTIFY_TIME_SYNC  %s",
+			IFMT "duration               %hu"
+			IFMT "NOTIFY_PORT_STATE      %s"
+			IFMT "NOTIFY_TIME_SYNC       %s"
+			IFMT "NOTIFY_PARENT_DATA_SET %s"
+			IFMT "NOTIFY_CMLDS           %s",
 			sen->duration,
 			event_bitmask_get(sen->bitmask, NOTIFY_PORT_STATE) ? "on" : "off",
-			event_bitmask_get(sen->bitmask, NOTIFY_TIME_SYNC) ? "on" : "off");
+			event_bitmask_get(sen->bitmask, NOTIFY_TIME_SYNC) ? "on" : "off",
+			event_bitmask_get(sen->bitmask, NOTIFY_PARENT_DATA_SET) ? "on" : "off",
+			event_bitmask_get(sen->bitmask, NOTIFY_CMLDS) ? "on" : "off");
 		break;
 	case MID_SYNCHRONIZATION_UNCERTAIN_NP:
 		mtd = (struct management_tlv_datum *) mgt->data;
@@ -621,6 +635,16 @@ static void pmc_show(struct ptp_message *msg, FILE *fp)
 			pwr->networkTimeInaccuracy,
 			pwr->totalTimeInaccuracy);
 		break;
+	case MID_CMLDS_INFO_NP:
+		cmlds = (struct cmlds_info_np *) mgt->data;
+		fprintf(fp, "CMLDS_INFO_NP "
+			IFMT "meanLinkDelay           %" PRId64
+			IFMT "scaledNeighborRateRatio %" PRId32
+			IFMT "as_capable              %" PRIu32,
+			cmlds->meanLinkDelay >> 16,
+			cmlds->scaledNeighborRateRatio,
+			cmlds->as_capable);
+		break;
 	case MID_LOG_ANNOUNCE_INTERVAL:
 		mtd = (struct management_tlv_datum *) mgt->data;
 		fprintf(fp, "LOG_ANNOUNCE_INTERVAL "
@@ -689,7 +713,7 @@ int main(int argc, char *argv[])
 	int ret = 0;
 	char line[1024], *command = NULL, uds_local[MAX_IFNAME_SIZE + 1];
 	enum transport_type transport_type = TRANS_UDP_IPV4;
-	UInteger8 boundary_hops = 1, domain_number = 0, transport_specific = 0;
+	UInteger8 boundary_hops = 1, domain_number = 0, transport_specific = 0, allow_unauth = 0;
 	struct ptp_message *msg;
 	struct option *opts;
 	struct config *cfg;
@@ -803,6 +827,11 @@ int main(int argc, char *argv[])
 	transport_type = config_get_int(cfg, NULL, "network_transport");
 	transport_specific = config_get_int(cfg, NULL, "transportSpecific") << 4;
 	domain_number = config_get_int(cfg, NULL, "domainNumber");
+	allow_unauth = config_get_int(cfg, NULL, "allow_unauth");
+
+	if (sad_create(cfg)) {
+		goto out;
+	}
 
 	if (!iface_name) {
 		if (transport_type == TRANS_UDS) {
@@ -821,8 +850,10 @@ int main(int argc, char *argv[])
 	print_set_syslog(1);
 	print_set_verbose(1);
 
-	pmc = pmc_create(cfg, transport_type, iface_name, boundary_hops,
-			 domain_number, transport_specific, zero_datalen);
+	pmc = pmc_create(cfg, transport_type, iface_name,
+			 config_get_string(cfg, NULL, "uds_address"),
+			 boundary_hops, domain_number, transport_specific,
+			 allow_unauth, zero_datalen);
 	if (!pmc) {
 		fprintf(stderr, "failed to create pmc\n");
 		config_destroy(cfg);
@@ -903,6 +934,7 @@ int main(int argc, char *argv[])
 	msg_cleanup();
 
 out:
+	sad_destroy(cfg);
 	config_destroy(cfg);
 	return ret;
 }
